package abacomayadecimale;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;

/**
 * Classe per l'implementazione di un JButton custom per la scelta del cursore.
 * Questo oggetto contiene 2 diversi cursori da poter utilizzare.
 */
public class CursorButton extends JButton {

    /**
     * Immagine del primo cursore.
     */
    private ImageIcon firstIcon;
    
    /**
     * Immagine del secondo cursore.
     */
    private ImageIcon secondIcon;
    
    /**
     * Primo cursore custom.
     */
    private Cursor firstCursor;

    /**
     * Secondo cursore custom.
     */
    private Cursor secondCursor;
    
    /**
     * Costruttore dell'oggetto CursorButton.
     * Viene create il JButton, vengono caricate le immagini e creati i cursori.
     * @param name nome dei due cursori (il secondo cursore avrà come nome immagine e nome cursore <code>name</code>+"Minus")
     * @param point coordinate all'interno dell'immagine del cursore in cui viene simulato il clic del cursore 
     */
    CursorButton(String name, Point point){
        super();
             
        //Carica l'immagine e crea il primo cursore
        firstIcon = new ImageIcon(this.getClass().getResource("images/"+name+".png"));
        firstCursor = Toolkit.getDefaultToolkit().createCustomCursor(firstIcon.getImage(), point, name);
        
        //Carica l'immagine e crea il secondo cursore
        secondIcon = new ImageIcon(this.getClass().getResource("images/"+name+"Minus.png"));
        secondCursor = Toolkit.getDefaultToolkit().createCustomCursor(secondIcon.getImage(), point, name+"Minus");
        
        //Imposta l'icona
        this.setIcon(firstIcon);
        //Imposta il bordo
        this.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        //Imposta la dimensione
        this.setPreferredSize(new Dimension(40, 40));
        //Non permette il mantenimento del focus
        this.setFocusable(false);        
    }
    
    /**
     * Ritorna il primo cursore.
     * @return ritorna il primo cursore
     */
    public Cursor getFirstCursor(){
        return firstCursor;
    }
    
    /**
     * Ritorna il secondo cursore.
     * @return ritorna il secondo cursore
     */
    public Cursor getSecondCursor(){
        return secondCursor;
    }

    /**
     * Cambia l'icona del JButton.
     * Se l'icona è quella del primo cursore, viene scambiata con quella del secondo, 
     * viceversa viene scambiata con quello del primo.
     */
    public void switchIcon(){
        if(this.getIcon() == firstIcon)
            this.setIcon(secondIcon);
        else this.setIcon(firstIcon);
    }
}