package abacomayadecimale;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

/**
 * Classe per l'implementazione della cella doppia nell'abaco.
 * La cella viene divisa a metà da una diagonale (secondaria), nella parte
 * superiore vengono inseriti oggetti che hanno valore decimale, mentre nella parte
 * inferiore vengono inseriti oggetti con valore unitario.
 */
public class DCell extends Cell implements MouseListener {
            
    /**
     * Larghezza e altezza della semicella.
     */
    private final static int semi = DIM/2;
    
    /**
     * Larghezza caracol.
     */
    private final static int wCara = 30;
    
    /**
     * Altezza caracol.
     */
    private final static int hCara = 18;
    
    /*+
     * Larghezza frijolito.
     */
    private final static int wFrij = 4;
    
    /**
     * Altezza frijolito.
     */
    private final static int hFrij = 4;
    
    /**
     * Larghezza palito.
     */
    private final static int wPali = 30;
    
    /**
     * Altezza palito.
     */
    private final static int hPali = 2;
    
    /**
     * Distanza tra gli oggetti (frijolito e palito).
     */
    private final static int dist = 3;
    
    /**
     * Numero di frijolito (punti) nella cella superiore.
     * Il frijolito ha valore 1.
     */
    private int upFrijolito;
    
    /**
     * Numero di palito nella cella superiore.
     * Il palito ha valore 5.
     */
    private int upPalito;
    
    /**
     * Boolean che permette la visualizzazione del Caracol nella cella inferiore.
     * Il caracol ha valore 0.
     */
    private boolean caracol;
    
    /**
     * Numero di frijolito nella cella inferiore.
     * Il frijolito ha valore 1.
     */
    private int downFrijolito;
    
    /**
     * Numero di palito nella cella inferiore.
     * Il palito ha valore 5.
     */
    private int downPalito;

    /**
     * Numero della diagonale che appare in basso a destra nella cella.
     */
    private int diagonalDown;
    
    /**
     * Numero della diagonale che appare in alto a sinistra nella cella.
     */
    private int diagonalUp;

    
    /**
     * Costruttore della Double Cell (DCell).
     * Viene creata una cella doppia che in modalità simulazione presenta il
     * numero della diagonale (se specificato nel costruttore) e in modalità 
     * esercitazione presenta la coordinata della cella (<code>row</code>|<code>col</code>)
     * in alto a sinistra.

     * <br>Per evitare di scrivere la diagonale superiore o inferiore basta passare
     * come parametro -1. Se non si vuole scrivere nessun numero nella cella, usare 
     * l'altro costruttore.
     * @param row riga in cui si trova la cella
     * @param col colonna in cui si trova la cella
     * @param diagonalUp numero che viene stampato in alto a sinistra. Per non
     * far stampare niente inserire -1
     * @param diagonalDown numero che viene stampato in basso a destra. Per non
     * far stampare niente inserire -1
     */
    public DCell(int row, int col, int diagonalUp, int diagonalDown){
        super(row, col);
        //Imposta i valori
        this.caracol = false;
        this.downFrijolito = 0;
        this.downPalito = 0;
        this.upFrijolito = 0;
        this.upPalito = 0;
        this.diagonalDown = diagonalDown;
        this.diagonalUp = diagonalUp;
        
        this.addMouseListener(this);
            
        this.update();
    }
    
    /**
     * Costruttore della Double Cell (DCell).
     * Viene creata una cella doppia che in modalità simulazione presenta il
     * numero della diagonale (se specificato nel costruttore) e in modalità esercitazione
     * presenta la coordinata della cella (<code>row</code>|<code>col</code>) in alto a sinistra.
     * 
     * @param row
     * @param col 
     */
    public DCell(int row, int col){
        this(row, col, -1, -1);
    }
 
    @Override
    public void paintComponent(Graphics g){
        super.paintComponent(g);
        
        g.setFont(Cell.FONT);
        
        if(AbacoFrame.currentModality == AbacoFrame.EXE){
            
            //Imposta lo sfondo
            g.setColor(Color.CYAN);
            g.fillRect(0, 0, DIM, DIM);
            
            g.setColor(Color.BLACK);
            //Disegna i numeri delle celle
            g.drawString((4-row)+"|"+(4-col), 2, 10);
            
        } else {
            g.setColor(Color.BLACK);
            //Disegna i numeri delle celle
            if(diagonalUp != -1)
                g.drawString(""+diagonalUp, 2, 10);
            if(diagonalDown != -1)
                g.drawString(""+diagonalDown, DIM-6, DIM-2);
        }
        
        //Disegna la diagonale di colore nero
        g.drawLine(DIM, 0, 0, DIM);

        //Disegna gli elementi della semicella superiore
        //Calcola i punti per disegnare gli elementi
        int x = (semi -(wFrij*upFrijolito) -(dist*(upFrijolito-1)))/2;
        int y = (semi-hFrij)/2 -(upPalito>0? (dist -hPali/2):0);
        
        //Disegna i Frijolito
        for(int i = 0; i<upFrijolito; i++){
            x += (i>0? (wFrij+dist):0);
            g.fillRect(x, y, wFrij, hFrij);    
        }
        
        //Disegna il Palito
        if(upPalito > 0){
            x = (semi - wPali)/2;
            y += (upFrijolito>0? (dist+hFrij):0);
            g.fillRect(x, y, wPali, hPali);
        }
        
        
        //Disegna gli elementi della semicella inferiore
        //Disegna il Caracol
        if(caracol){
            int xCaracol = semi + (semi-wCara)/2;
            int yCaracol = semi + (semi-hCara)/2;
            
            g.drawOval(xCaracol, yCaracol, wCara, hCara);           //Ovale esterno 
            g.drawArc( xCaracol+6, yCaracol+1, 8, 16, 90, 180);     //Primo arco
            g.drawArc(xCaracol+13, yCaracol+0, 8, 18, 90, 180);     //Secondo arco
            g.drawArc(xCaracol+20, yCaracol+1, 8, 15, 85, 180);     //Terzo arco
            return;
        }
        
        //Calcola i punti per disegnare gli elementi
        x = semi + (semi -(wFrij*downFrijolito) -(dist*(downFrijolito-1)))/2;
        y = semi + (semi-hFrij)/2 -(downPalito>0? (dist +hPali/2):0);

        //Disegna i frijolito
        for(int i = 0; i<downFrijolito; i++){
            x += (i>0? (wFrij+dist):0);
            g.fillRect(x, y, wFrij, hFrij);    
        }
        
        //Disegna il palito
        if(downPalito > 0){
            x = semi + (semi - wPali)/2;
            y += (downFrijolito>0? (dist+hFrij):0);
            g.fillRect(x, y, wPali, hPali);
        }
    }

    /**
     * Ritorna il numero di frijolito contenuti nella semicella inferiore.
     * @return numero di frijolito contenuti nella semicella inferiore
     */
    public int getDownFrijolito(){
        return downFrijolito;
    }
    
    /**
     * Ritorna il numero di palito contenuti nella semicella inferiore.
     * @return numero di palito contenuti nella semicella inferiore
     */
    public int getDownPalito(){
        return downPalito;
    }
    
    /**
     * Ritorna il numero di frijolito contenuti nella semicella superiore.
     * @return numero di frijolito contenuti nella semicella superiore
     */
    public int getUpFrijolito(){
        return upFrijolito;
    }
    
    /**
     * Ritorna il numero di palito contenuti nella semicella superiore.
     * @return numero di palito contenuti nella semicella superiore
     */
    public int getUpPalito(){
        return upPalito;
    }
    
    @Override
    public int getValue(){
        return this.getUpValue()*10 + this.getDownValue();
    }
    
    /**
     * Ritorna il valore decimale dell'unione degli elmenti della semicella inferiore.
     * @return un intero con il valore decimale della semicella inferiore.
     */
    public int getDownValue(){
        return downPalito*5 + downFrijolito;
    }
    
    /**
     * Ritorna il valore decimale dell'unione degli elmenti della semicella superiore.
     * @return un intero con il valore decimale della semicella superiore.
     */
    public int getUpValue(){
        return upPalito*5 + upFrijolito;
    }
    
    @Override
    public void reset(){
        caracol = false;
        downFrijolito = 0;
        downPalito = 0;
        upFrijolito = 0;
        upPalito = 0;
        
        this.update();
    }

    /**
     * Imposta il numero di frijolito nella semicella inferiore.
     * @param downFrijolito valore con il numero di frijolito da inserire nella
     * semicella inferiore
     */
    public void setDownFrijolito(int downFrijolito){
        this.downFrijolito = downFrijolito;
        caracol = this.getDownValue() == 0;
        this.update();
    }
    
    /**
     * Imposta il numero di palito nella semicella inferiore.
     * @param downPalito valore con il numero di palito da inserire nella
     * semicella inferiore
     */
    public void setDownPalito(int downPalito){
        this.downPalito = downPalito;
        caracol = this.getDownValue() == 0;
        this.update();
    }
    
    /**
     * Imposta il numero di frijolito nella semicella superiore.
     * @param upFrijolito valore con il numero di frijolito da inserire nella
     * semicella superiore
     */
    public void setUpFrijolito(int upFrijolito){
        this.upFrijolito = upFrijolito;
        this.update();
    }
    
   /**
     * Imposta il numero di palito nella semicella superiore.
     * @param upPalito valore con il numero di palito da inserire nella
     * semicella superiore
     */
    public void setUpPalito(int upPalito){
        this.upPalito = upPalito;
        this.update();
    }

    @Override
    public void setValue(int x){
        
        int deca = x/10;    //Decine
        int unities = x%10; //Unità

        //Imposta le decine nella semicella superiore.
        this.upFrijolito = deca%5;
        this.upPalito = deca/5;
        
        //Imposta le unità nella semicella inferiore.
        this.downFrijolito = unities%5;
        this.downPalito = unities/5;
        
        caracol = this.getDownValue() == 0;
        
        this.update();
    } 
    
    /**
     * Gestisce il clic da parte dell'utente con i vari cursori nella semicella
     * inferiore.
     */
    public void clickDown(){
        
        int value = this.getValue();

        switch(this.getCursor().getName()){

            case "cross":
                caracol = false;
                downFrijolito = 0;
                downPalito = 0;
                break;

            case "caracol":
                caracol = true;
                downFrijolito = 0;
                downPalito = 0;
                break;

            case "frijolito":
                caracol = false;
                if(value+1 < 100)
                    this.setValue(value+1);
                break;

            case "palito":
                caracol = false;
                if(value+5 < 100)
                    this.setValue(value+5);
                break;
                
            case "crossMinus":
                caracol = false;
                downFrijolito = 0;
                downPalito = 0;
                break;

            case "caracolMinus":
                if(caracol == true)
                    caracol = false;
                break;

            case "frijolitoMinus":
                if(value-1 >= 0)
                    this.setValue(value-1);

                break;

            case "palitoMinus":
                if(downPalito > 0)
                    downPalito--;
                break;
        }

    }
    
    
    /**
     * Gestisce il clic da parte dell'utente con i vari cursori nella semicella
     * superiore.
     */
    public void clickUp(){
        
        int value = this.getValue();
        
        switch(this.getCursor().getName()){

            case "cross":
                upFrijolito = 0;
                upPalito = 0;
                break;

            case "caracol":
                break;

            case "frijolito":
                if(value+10 < 100)
                    this.setValue(value+10);
                break;

            case "palito":
                if(value+50 < 100)
                    this.setValue(value+50);
                break;
                
            case "crossMinus":
                upFrijolito = 0;
                upPalito = 0;
                break;

            case "caracolMinus":
                break;

            case "frijolitoMinus":
                if(this.getValue()-10 >= 0)
                    this.setValue(value-10);
                break;

            case "palitoMinus":
                if(upPalito > 0)
                    upPalito--;
                break;
        }
    }

    @Override
    public boolean isCliccable(int modality){
        return modality == AbacoFrame.currentModality;
    }
    
    /**
     * Ascoltatore utilizzato per la pressione del mouse.
     * In questo metodo si controlla in quale delle semicelle l'utente ha cliccato
     * e richiama i metodi per gestire l'evento.
     * @see DCell#clickDown() 
     * @see DCell#clickUp()
     */
    @Override
    public void mousePressed(MouseEvent e) {
        if(this.isCliccable(AbacoFrame.EXE)){
            if(e.getX() + e.getY() > DIM)
                this.clickDown();
            else this.clickUp();
        }
    }    

    @Override
    public void mouseClicked(MouseEvent e) {
    }

    @Override
    public void mouseReleased(MouseEvent e) {
    }

    @Override
    public void mouseEntered(MouseEvent e) {
    }

    @Override
    public void mouseExited(MouseEvent e) {
    }
}