package abacomayadecimale;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import javax.swing.JPanel;

/**
 * Classe per l'implementazione di un ToolTip custom.
 * Viene creato un piccolo pannello di colore arancio in cui vi è scritto
 * "Valore da inserire: " e in aggiunta il valore passato nel costruttore.
 */
public class Tip extends JPanel {
    
    /**
     * Valore che viene suggerito nel ToolTip.
     */
    private int value;
    
    /**
     * Costruttore dell'oggetto Tip.
     * Viene ricevuto il valore da suggerire all'utente e il punto in cui il
     * ToolTip deve essere posizionato nel JLayeredPane del JFrame.
     * 
     * @param value valore da suggerire
     * @param point punto in cui deve essere posizionato
     */
    Tip(int value, Point point){
        super();
        this.value = value;
        
        //Imposta la larghezza del pannello in base al numero di cifre da visualizzare
        if(value < 10)
            this.setSize(new Dimension(110, 20));
        else this.setSize(new Dimension(115, 20));
        
        //Posiziono l'oggetto
        this.setLocation(point);
    }
    
    @Override
    public void paintComponent(Graphics g){
        super.paintComponent(g);
        
        //Sfondo
        g.setColor(Color.ORANGE);
        g.fillRect(0, 0, 115, 20);
        
        //Stringa da visualizzare
        g.setColor(Color.BLACK);
        g.setFont(new Font("Lucida Sans", Font.PLAIN, 10));
        g.drawString("Valore da inserire : "+value, 5, 13);
    }
}
